#include "Robojax_Energy_Meter.h"
#include <inttypes.h>
#include <Arduino.h>
#include <Wire.h>

// When the display powers up, it is configured as follows:
//
// 1. Display clear
// 2. Function set:
//    DL = 1; 8-bit interface data
//    N = 0; 1-line display
//    F = 0; 5x8 dot character font
// 3. Display on/off control:
//    D = 0; Display off
//    C = 0; Cursor off
//    B = 0; Blinking off
// 4. Entry mode set:
//    I/D = 1; Increment by 1
//    S = 0; No shift
//
// Note, however, that resetting the Arduino doesn't reset the LCD, so we
// can't assume that its in that state when a sketch starts (and the
// LiquidCrystal constructor is called).


/*
 * calculateCurrent()
 * @brief calculates the current
 * @param none

 * @return none
 * Written by Ahmad Shamshiri for robojax.com
 * on May 23, 2020 at 12:02 in Ajax, Ontario, Canada
 */
 
 	//Constructor for Current Measurement
Robojax_Energy_Meter::Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, 
							 uint8_t shuntInPin, uint8_t shuntCurrent, uint8_t shuntVoltage,
							 uint8_t correctionValue, uint8_t iteration, float voltageReference,
							 uint8_t bitResolution, float minimumCurrent, bool debugOnce,
						uint8_t charsize = LCD_5x8DOTS)
{
	analogReference(DEFAULT);//1.1V internal reference
	_addr = lcd_addr;
	_cols = lcd_cols;
	_rows = lcd_rows;
	_charsize = charsize;
	_backlightval = LCD_BACKLIGHT;
	_shuntInPin = shuntInPin;
	_shuntCurrent = shuntCurrent;
	_shuntVoltage = shuntVoltage;
	_correctionValue =correctionValue;
	_iteration = iteration;
	_voltageReference = voltageReference;
	_bitResolution = bitResolution;
	_minimumCurrent = minimumCurrent;
	_debugOnce = debugOnce;
	_is_energy =false;
	
}

	//constructor for Energy Calculation
Robojax_Energy_Meter::Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, 
							 uint8_t shuntInPin, uint8_t shuntCurrent, uint8_t shuntVoltage,
							 uint8_t correctionValue, uint8_t iteration, float voltageReference,
							 uint8_t bitResolution, float minimumCurrent, bool debugOnce,
							 float ValueR1, float ValueR2, uint8_t voltageInPin, uint8_t pushButtonPin,
						uint8_t charsize = LCD_5x8DOTS)
{
	analogReference(DEFAULT);//1.1V internal reference
	_addr = lcd_addr;
	_cols = lcd_cols;
	_rows = lcd_rows;
	_charsize = charsize;
	_backlightval = LCD_BACKLIGHT;
	_shuntInPin = shuntInPin;
	_shuntCurrent = shuntCurrent;
	_shuntVoltage = shuntVoltage;
	_correctionValue =correctionValue;
	_iteration = iteration;
	_voltageReference = voltageReference;
	_bitResolution = bitResolution;
	_minimumCurrent = minimumCurrent;
	_debugOnce = debugOnce;
	_is_energy =true;
	
	_ValueR2 = ValueR2;
	_ValueR1 = ValueR1;
	_voltageInPin = voltageInPin;
	_pushButtonPin = pushButtonPin;
	_seconds = 0;
	_resetTime =0;

	
    pinMode(_pushButtonPin, INPUT_PULLUP);//for energy reset push button	
}//Robojax_Energy_Meter						

void Robojax_Energy_Meter::calculateCurrent()
{
 //robojax.com 500A Shunt Current Measurement for Arduino

    float averageSensorValue =0;
    int sensorValue ;
    float voltage, current;

    for(int i=0; i< _iteration; i++)
    {   
      sensorValue = analogRead(_shuntInPin)+_correctionValue;//
	  //Serial.println(sensorValue);
	  
      delay(5);
      if(sensorValue !=0 && sensorValue < 100)
      {
        voltage = (sensorValue) * (_voltageReference /  (pow(2,_bitResolution)-1)); 
		  
        current  = voltage * (_shuntCurrent /_shuntVoltage )  ;
		
        if(i !=0){
          averageSensorValue += current;
        }
        delay(1);
      }
    }    
	
   
    averageSensorValue /=(_iteration-1);//we have ignored 1st reading, so take average one less
	if(_minimumCurrent <= averageSensorValue )
	{		  
		_currentValue = averageSensorValue;
		//while(1);
	}else{
		_currentValue = 0;	
	}
   
	
}//calculateCurrent()



/*
 * showCurrent()
 * @brief prints  current on LCD1602 or LCD2004
 * @param current of type float
 * @return none
 * Written by Ahmad Shamshiri for robojax.com
 * on May 23, 2020 at 14:40 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::showCurrent()
{
   // Robojax.com LCD1602 for 75mV Shune Current Demo
	//lcd.setCursor(x,y);
	//lcd.write(254);
  int current = round(getCurrent());
   setCursor (CURRENT_TITLE_POSITION_CHAR,CURRENT_POSITION_ROW); //
   if(_is_energy){
		print(CURRENT_TITLE); 
	  for (int i=2; i<=9; i++)
	  {
		setCursor(i ,CURRENT_POSITION_ROW);
		write(254);
	  } 
		setCursor (CURRENT_VALUE_POSITION_CHAR,CURRENT_POSITION_ROW); //	
		print(current);    
   }else{
		print(CURRENT_TITLE2); 	
	  for (int i=9; i<=19; i++)
	  {
		setCursor(i ,CURRENT_POSITION_ROW);
		write(254);
	  }  
		setCursor (CURRENT_VALUE2_POSITION_CHAR,CURRENT_POSITION_ROW); //	
		print(current);  	  
   }


   print(CURRENT_UNIT);

 // Robojax.com LCD1602 for 75mV Shune Current Demo
}//showCurrent()  


/*
 * getCurrent()
 * @brief returns the current
 * @param none
 * @return _currentValue
 * Written by Ahmad Shamshiri for robojax.com
 * on May 23, 2020 at 11:57 in Ajax, Ontario, Canada
 */
float Robojax_Energy_Meter::getCurrent()
{
	calculateCurrent();
	return _currentValue;
}//getCurrent()





/*
 * @brief calculates the input voltage and updates the variable "voltage"
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 15:21 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::readVoltage(){

    //int A0Value = analogRead(_voltageInPin);
  //Serial.print("sensed Value:");
  //Serial.println(A0Value); 	
    float voltage_temp_average=0;
	float voltage_sensed;
    for(int i=0; i < _iteration; i++)
    {
		
		voltage_sensed = (analogRead(_voltageInPin)+_correctionValue) * (_voltageReference / (pow(2,_bitResolution)-1)); 
 		
		voltage_temp_average += voltage_sensed;
    }
      
   voltage_temp_average /= _iteration;
  

	voltage_temp_average /=1000;
     
  _inputVoltage = voltage_temp_average * ( 1 + ( (float) _ValueR2 /  (float) _ValueR1) );
 
 
  //Robojax.com ARDVC-01 Measure any voltage with Arduino 
}//readVoltage()


/*
 * @brief displays the duration of power calcuaotion
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 24, 2020 at 00:28 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::showDuration(){
   float totalHours = (float)_seconds/3600.0;
   
   setCursor (DURATION_TITLE_POSITION_CHAR,DURATION_POSITION_ROW); //
   print(DURATION_TITLE); 
	  for (int i=1; i<=5; i++)
	  {
		setCursor(DURATION_VALUE_POSITION_CHAR +i ,DURATION_POSITION_ROW);
		write(254);
	  }

   setCursor (DURATION_VALUE_POSITION_CHAR,DURATION_POSITION_ROW); //
   print(secondsToTime()); 
   setCursor (DURATION_UNIT_POSITION_CHAR,DURATION_POSITION_ROW); //   
   print(DURATION_UNIT);


}//showDuration()

/*
 * @brief calculates day:Hour:minutes:seconds from integer
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 25, 2020 at 07:40 in Ajax, Ontario, Canada
 */
String Robojax_Energy_Meter::secondsToTime(){

   uint16_t hours, days, minutes, seconds;
   unsigned long SID= 24L*60L*60L;//seconds in day
   unsigned long SIH= 60L*60L;//seconds in hour
   unsigned long t = _seconds;//98467543=1139:16:5:43 DHMS
   
   days = t/SID;  
   hours   =  (t % SID) / SIH;//from remainders of seconds of a day get hours
   minutes = ((t % SID)%SIH)/60L;
   seconds = ((t % SID)%SIH)%60L;
   String full =String(days) +":"+String(hours)+":"+String(minutes)+":"+String(seconds);
   return full;
}//showDuration()


/*
 * @brief calculates the energy
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 24, 2020 at 00:21 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::calculateEnergy(){
			
	if(digitalRead(_pushButtonPin) == LOW)
	{
		clearLine(3);
		_resetTime = millis() /1000;
		_seconds =0;//reset the time when reset push button is pressed
		_energy  =0;//reset the energy when reset push button is pressed
		setCursor (0,3); //
		print(ENERGY_CLEAR_MESSAGE); 	
        delay(2000);
		clearLine(3);
	}

	_seconds = (millis()/1000) - _resetTime;	
	//Serial.print("_resetTime: ");Serial.println(_resetTime);
	//Serial.println(millis()/1000);
	
     _energy += (_inputVoltage * _currentValue)/(3600);
   
}//calculateEnergy()



/*
 * @brief returns the energy
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 18:37 in Ajax, Ontario, Canada
 */
float Robojax_Energy_Meter::getEnergy(){
	calculateEnergy();
    return _energy;
}//getEnergy()

/*
 * @brief displays the energy
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 18:37 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::showEnergy(){
	showPower();
	getEnergy();
	showDuration();
   setCursor (ENERGY_TITLE_POSITION_CHAR,ENERGY_POSITION_ROW); //
   print(ENERGY_TITLE); 
	  for (int i=1; i<=5; i++)
	  {
		setCursor(ENERGY_VALUE_POSITION_CHAR +i ,ENERGY_POSITION_ROW);
		write(254);
	  }

   setCursor (ENERGY_VALUE_POSITION_CHAR,ENERGY_POSITION_ROW); //
   print(_energy); 
   //setCursor (ENERGY_UNIT_POSITION_CHAR,ENERGY_POSITION_ROW); //   
   print(ENERGY_UNIT);
  
}//showEnergy()


/*
 * @brief calculates the power
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 18:37 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::calculatePower(){
     _power = (int)_inputVoltage * (int)_currentValue;
   
}//calculatePower()



/*
 * @brief returns the power
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 18:37 in Ajax, Ontario, Canada
 */
float Robojax_Energy_Meter::getPower(){
	calculatePower();
    return _power;
}//getPower()

/*
 * @brief displays the power
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 18:37 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::showPower(){
	calculatePower();
   setCursor (POWER_TITLE_POSITION_CHAR,POWER_POSITION_ROW); //
   print(POWER_TITLE); 
	  for (int i=1; i<=5; i++)
	  {
		setCursor(POWER_VALUE_POSITION_CHAR +i ,POWER_POSITION_ROW);
		write(254);
	  }

   setCursor (POWER_VALUE_POSITION_CHAR,POWER_POSITION_ROW); //
   print(_power); 
   //setCursor (POWER_UNIT_POSITION_CHAR,POWER_POSITION_ROW); //   
   print(POWER_UNIT);

}//showshowPower()

/*
   clearLine(uint8_t line)
 * @brief clears a line of display (erases all characters)
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 28, 2020 at 16:21 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::clearLine(uint8_t line)
{
	  for (int i=0; i<_cols; i++)
	  {
		setCursor (i,line); //	
		write(254);
	  }	

}//clearLine(uint8_t line)

/*
 * @brief calculates the input voltage and updates the variable "voltage"
 * @param none
 * @return does not return anything
 * Written by Ahmad Shamshiri
 * www.Robojax.com code May 23, 2020 at 15:21 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::showVoltage(){
	readVoltage();
   setCursor(VOLTAGE_TITLE_POSITION_CHAR,VOLTAGE_POSITION_ROW); //
   print(VOLTAGE_TITLE); 	
	  for (int i=VOLTAGE_VALUE_POSITION_CHAR; i<_cols; i++)
	  {
		setCursor(i ,VOLTAGE_POSITION_ROW);
		write(254);
	  }   
   setCursor (VOLTAGE_VALUE_POSITION_CHAR,VOLTAGE_POSITION_ROW); //
   print(_inputVoltage); 
   //setCursor (VOLTAGE_UNIT_POSITION_CHAR,VOLTAGE_POSITION_ROW); //   
   print(VOLTAGE_UNIT);
   
}// showVoltage()
	
/*
 * printSerial()
 * @brief prints  the current on serial monitor
 * @param none
 * @return none
 * Written by Ahmad Shamshiri for robojax.com
 * on May 08, 2020 at 02:45 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::printSerial()
{
   Serial.println("Current:");
   Serial.print(_currentValue);
   Serial.println("A");
if(_is_energy)   
{
   Serial.print("Power:");
   Serial.print(_currentValue*_inputVoltage);
   Serial.println("W");	
   
   Serial.print("Voltage:");
   Serial.print(_inputVoltage);
   Serial.println("V");	  
   
   Serial.print("Energy:");
   Serial.print(_energy);
   Serial.println("Wh");	
 
   Serial.print("Time:");
   Serial.print(secondsToTime());
   Serial.println(" Days, Hours, Minutes, Seconds");   
}
   Serial.println();
   
 
}//printSerial()


/*
 * printDebug()
 * @brief prints  the full details of current measurement
 * @param none
 * @return none
 * Written by Ahmad Shamshiri for robojax.com
 * on May 23, 2020 at 13:10 in Ajax, Ontario, Canada
 */
void Robojax_Energy_Meter::printDebug()
{
  //robojax.com 500A Shunt Current Measurement for Arduino
    Serial.print("Debug Details Measureing ");
    Serial.print(_iteration);
    Serial.println(" Times");
    Serial.print("Reading from pin: \t");
    Serial.println(_shuntInPin); 
    
    Serial.print("Shunt Voltage: \t\t");
    Serial.print(_shuntVoltage);Serial.println("mV"); 
    Serial.print("Shunt Current: \t\t");
    Serial.print(_shuntCurrent);Serial.println("A"); 
    Serial.print("Voltage Reference: \t");
    Serial.print(_voltageReference);Serial.println("mV"); 
    Serial.print("Bits Resolution: \t");
    Serial.print(_bitResolution);Serial.println(" bits"); 
    Serial.print("Correction Value: \t");
    Serial.print(_correctionValue);Serial.println(" unit"); 
if(_debugOnce){	
    Serial.print("Debug ONCE: \t\t");
    Serial.println("YES"); 	
}else{
    Serial.print("Debug ONCE: \t\t");
    Serial.println("NO"); 		
}	
if(_is_energy){
     Serial.println("===Energy Related Settings");
    Serial.print("R1: \t");
    Serial.print(_ValueR1); Serial.println("Ω");
    Serial.print("R2: \t");
    Serial.print(_ValueR2); Serial.println("Ω");	
    Serial.print(_maxVoltage/1000); Serial.println("V"); 	 
	_maxVoltage = _voltageReference* ( 1 + ( (float) _ValueR2 /  (float) _ValueR1) );
    Serial.print("Maximum Voltage: \t");
    Serial.print(_maxVoltage/1000); Serial.println("V"); 
	
}
    
    Serial.println();        
    Serial.println("Sensor\tVoltage\t\tCurrent");    
    int sensorValue ;
    float voltage, current;
    float averageCurrentValue=0;

    for(int i=0; i< _iteration; i++)
    {   
      sensorValue = analogRead(_shuntInPin)+_correctionValue;//
      delay(5);
      if(sensorValue !=0 && sensorValue < 100)
      {      
        Serial.print(sensorValue);
        voltage = (sensorValue) * (_voltageReference /  (pow(2,_bitResolution)-1)); 
        Serial.print("\t");Serial.print(voltage);  
        current    = voltage * (_shuntCurrent /_shuntVoltage )  ;
        if(i !=0) averageCurrentValue += current;
        Serial.print("mV\t\t");Serial.print(current);Serial.println("A");                    
        delay(100);
      }else{
        i--;
      }
    }    
    averageCurrentValue /=(_iteration-1);//we have ignored 1st reading, so take average one less
	if(_minimumCurrent <= averageCurrentValue )
	{
		_currentValue = averageCurrentValue;
	}else{
		_currentValue = 0;	
	}
    Serial.print("Average Current: ");   
    Serial.print(_currentValue);            
    Serial.println("A");  
    Serial.println();   
    while(_debugOnce);
}//printDebug

	
 
//constructor
Robojax_Energy_Meter::Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, uint8_t charsize)
{
	_addr = lcd_addr;
	_cols = lcd_cols;
	_rows = lcd_rows;
	_charsize = charsize;
	_backlightval = LCD_BACKLIGHT;
}


void Robojax_Energy_Meter::begin() {
	Wire.begin();
	_displayfunction = LCD_4BITMODE | LCD_1LINE | LCD_5x8DOTS;

	if (_rows > 1) {
		_displayfunction |= LCD_2LINE;
	}

	// for some 1 line displays you can select a 10 pixel high font
	if ((_charsize != 0) && (_rows == 1)) {
		_displayfunction |= LCD_5x10DOTS;
	}

	// SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!
	// according to datasheet, we need at least 40ms after power rises above 2.7V
	// before sending commands. Arduino can turn on way befer 4.5V so we'll wait 50
	delay(50);

	// Now we pull both RS and R/W low to begin commands
	expanderWrite(_backlightval);	// reset expanderand turn backlight off (Bit 8 =1)
	delay(1000);

	//put the LCD into 4 bit mode
	// this is according to the hitachi HD44780 datasheet
	// figure 24, pg 46

	// we start in 8bit mode, try to set 4 bit mode
	write4bits(0x03 << 4);
	delayMicroseconds(4500); // wait min 4.1ms

	// second try
	write4bits(0x03 << 4);
	delayMicroseconds(4500); // wait min 4.1ms

	// third go!
	write4bits(0x03 << 4);
	delayMicroseconds(150);

	// finally, set to 4-bit interface
	write4bits(0x02 << 4);

	// set # lines, font size, etc.
	command(LCD_FUNCTIONSET | _displayfunction);

	// turn the display on with no cursor or blinking default
	_displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
	display();

	// clear it off
	clear();

	// Initialize to default text direction (for roman languages)
	_displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;

	// set the entry mode
	command(LCD_ENTRYMODESET | _displaymode);

	home();

}

/********** high level commands, for the user! */
void Robojax_Energy_Meter::clear(){
	command(LCD_CLEARDISPLAY);// clear display, set cursor position to zero
	delayMicroseconds(2000);  // this command takes a long time!
}

void Robojax_Energy_Meter::home(){
	command(LCD_RETURNHOME);  // set cursor position to zero
	delayMicroseconds(2000);  // this command takes a long time!
}

void Robojax_Energy_Meter::setCursor(uint8_t col, uint8_t row){
	int row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };
	if (row > _rows) {
		row = _rows-1;    // we count rows starting w/0
	}
	command(LCD_SETDDRAMADDR | (col + row_offsets[row]));
}

// Turn the display on/off (quickly)
void Robojax_Energy_Meter::noDisplay() {
	_displaycontrol &= ~LCD_DISPLAYON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void Robojax_Energy_Meter::display() {
	_displaycontrol |= LCD_DISPLAYON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turns the underline cursor on/off
void Robojax_Energy_Meter::noCursor() {
	_displaycontrol &= ~LCD_CURSORON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void Robojax_Energy_Meter::cursor() {
	_displaycontrol |= LCD_CURSORON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turn on and off the blinking cursor
void Robojax_Energy_Meter::noBlink() {
	_displaycontrol &= ~LCD_BLINKON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void Robojax_Energy_Meter::blink() {
	_displaycontrol |= LCD_BLINKON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// These commands scroll the display without changing the RAM
void Robojax_Energy_Meter::scrollDisplayLeft(void) {
	command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}
void Robojax_Energy_Meter::scrollDisplayRight(void) {
	command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

// This is for text that flows Left to Right
void Robojax_Energy_Meter::leftToRight(void) {
	_displaymode |= LCD_ENTRYLEFT;
	command(LCD_ENTRYMODESET | _displaymode);
}

// This is for text that flows Right to Left
void Robojax_Energy_Meter::rightToLeft(void) {
	_displaymode &= ~LCD_ENTRYLEFT;
	command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'right justify' text from the cursor
void Robojax_Energy_Meter::autoscroll(void) {
	_displaymode |= LCD_ENTRYSHIFTINCREMENT;
	command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'left justify' text from the cursor
void Robojax_Energy_Meter::noAutoscroll(void) {
	_displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
	command(LCD_ENTRYMODESET | _displaymode);
}

// Allows us to fill the first 8 CGRAM locations
// with custom characters
void Robojax_Energy_Meter::createChar(uint8_t location, uint8_t charmap[]) {
	location &= 0x7; // we only have 8 locations 0-7
	command(LCD_SETCGRAMADDR | (location << 3));
	for (int i=0; i<8; i++) {
		write(charmap[i]);
	}
}

// Turn the (optional) backlight off/on
void Robojax_Energy_Meter::noBacklight(void) {
	_backlightval=LCD_NOBACKLIGHT;
	expanderWrite(0);
}

void Robojax_Energy_Meter::backlight(void) {
	_backlightval=LCD_BACKLIGHT;
	expanderWrite(0);
}
bool Robojax_Energy_Meter::getBacklight() {
  return _backlightval == LCD_BACKLIGHT;
}


/*********** mid level commands, for sending data/cmds */

inline void Robojax_Energy_Meter::command(uint8_t value) {
	send(value, 0);
}

inline size_t Robojax_Energy_Meter::write(uint8_t value) {
	send(value, Rs);
	return 1;
}


/************ low level data pushing commands **********/

// write either command or data
void Robojax_Energy_Meter::send(uint8_t value, uint8_t mode) {
	uint8_t highnib=value&0xf0;
	uint8_t lownib=(value<<4)&0xf0;
	write4bits((highnib)|mode);
	write4bits((lownib)|mode);
}

void Robojax_Energy_Meter::write4bits(uint8_t value) {
	expanderWrite(value);
	pulseEnable(value);
}

void Robojax_Energy_Meter::expanderWrite(uint8_t _data){
	Wire.beginTransmission(_addr);
	Wire.write((int)(_data) | _backlightval);
	Wire.endTransmission();
}

void Robojax_Energy_Meter::pulseEnable(uint8_t _data){
	expanderWrite(_data | En);	// En high
	delayMicroseconds(1);		// enable pulse must be >450ns

	expanderWrite(_data & ~En);	// En low
	delayMicroseconds(50);		// commands need > 37us to settle
}

void Robojax_Energy_Meter::load_custom_character(uint8_t char_num, uint8_t *rows){
	createChar(char_num, rows);
}

void Robojax_Energy_Meter::setBacklight(uint8_t new_val){
	if (new_val) {
		backlight();		// turn backlight on
	} else {
		noBacklight();		// turn backlight off
	}
}

void Robojax_Energy_Meter::printstr(const char c[]){
	//This function is not identical to the function used for "real" I2C displays
	//it's here so the user sketch doesn't have to be changed
	print(c);
}
